@@ -1,7 +1,7 @@
 /*
  * This file is part of the recoverjpeg program.
  *
- * Copyright (c) 2004 Samuel Tardieu <sam@rfc1149.net>
+ * Copyright (c) 2004-2010 Samuel Tardieu <sam@rfc1149.net>
  * http://www.rfc1149.net/devel/recoverjpeg
  *
  * recoverjpeg is released under the GNU General Public License
@@ -22,6 +22,8 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "utils.h"
+
 static int verbose = 0;
 static int quiet = 0;
 static size_t max_size = 6*1024*1024;
@@ -55,7 +57,7 @@ static void
 display_progressbar (off_t offset, unsigned int n)
 {
   off_t to_display;
-  static int old_n = -1;
+  static unsigned int old_n = -1;
   static int gib_mode = 0;
   static off_t old_to_display = 0.0;
 
@@ -111,7 +113,7 @@ jpeg_size (unsigned char *start)
     if (code == 0xd9) {
       if (verbose) {
 	fprintf (stderr,
-		 "   Found end of image after %d bytes\n", addr-start+1);
+		 "   Found end of image after %ld bytes\n", (long)(addr-start+1));
       }
       return addr-start;
     }
@@ -128,7 +130,7 @@ jpeg_size (unsigned char *start)
     addr += size;
 
     if (verbose) {
-      fprintf (stderr, "   Found section %02x of len %d\n", code, size);
+      fprintf (stderr, "   Found section %02x of len %ld\n", code, (long)size);
     }
 
     if (size < 2 || size > max_size) {
@@ -145,13 +147,13 @@ jpeg_size (unsigned char *start)
       }
 
       for (;
-	   addr-start < max_size &&
+	   (size_t) (addr-start) < max_size &&
 	     (*addr != 0xff ||
 	      *(addr+1) == 0 ||                            /* Escape */
 	      (*(addr + 1) >= 0xd0 && *(addr + 1) <= 0xd7) /* RSTn */);
 	   addr++);
 
-      if (addr - start >= max_size) {
+      if ((size_t) (addr - start) >= max_size) {
 	if (verbose) {
 	  fprintf (stderr, "too big, aborting\n");
 	}
@@ -159,40 +161,13 @@ jpeg_size (unsigned char *start)
       }
 
       if (verbose) {
-	fprintf (stderr, "found at offset %d\n", addr - start);
+	fprintf (stderr, "found at offset %ld\n", (long)(addr - start));
       }
 
     }
   }
 }
 
-static unsigned int
-atoi_suffix (char *arg)
-{
-  int multiplier = 1;
-
-  switch (arg[strlen(arg)-1]) {
-  case 'g':
-  case 'G':
-    multiplier = 1024*1024*1024;
-    break;
-  case 'm':
-  case 'M':
-    multiplier = 1024*1024;
-    break;
-  case 'k':
-  case 'K':
-    multiplier = 1024;
-    break;
-  }
-
-  if (multiplier != 1) {
-    arg[strlen(arg)-1] = '\0';
-  }
-
-  return atoi (arg) * multiplier;
-}
-
 int
 main (int argc, char *argv[])
 {
@@ -204,7 +179,7 @@ main (int argc, char *argv[])
   char buffer[100];
   int page_size;
   off_t offset;
-  unsigned char *format;
+  char *format;
   int c;
 
   read_size = 128*1024*1024;
@@ -215,7 +190,7 @@ main (int argc, char *argv[])
   while ((c = getopt (argc, argv, "b:f:hi:m:qr:v")) != -1) {
     switch (c) {
     case 'b':
-      block_size = atoi_suffix (optarg);
+      block_size = atol_suffix (optarg);
       break;
     case 'f':
       format = optarg;
@@ -224,13 +199,13 @@ main (int argc, char *argv[])
       begin_index = atoi (optarg);
       break;
     case 'm':
-      max_size = atoi_suffix (optarg);
+      max_size = atol_suffix (optarg);
       break;
     case 'q':
       quiet = 1;
       break;
     case 'r':
-      read_size = atoi_suffix (optarg);
+      read_size = atol_suffix (optarg);
       break;
     case 'v':
       verbose = 1;
@@ -262,8 +237,8 @@ main (int argc, char *argv[])
     }
     read_size = (read_size + page_size - 1) / page_size * page_size;
     if (!quiet) {
-      fprintf (stderr, "Adjusted read size to %u bytes\n",
-	       read_size);
+      fprintf (stderr, "Adjusted read size to %ld bytes\n",
+	       (long)read_size);
     }
   }
 
@@ -281,9 +256,9 @@ main (int argc, char *argv[])
       display_progressbar (offset, i);
     }
 
-    if (addr == NULL || (start + read_size - addr) < max_size) {
+    if (addr == NULL || (size_t) (start + read_size - addr) < max_size) {
       off_t base_offset;
-      size_t n;
+      long n;
 
       base_offset = offset / page_size * page_size;
 
@@ -305,15 +280,15 @@ main (int argc, char *argv[])
       snprintf (buffer, sizeof buffer, format, begin_index + i);
       i++;
       if (verbose) {
-	printf ("%s %d bytes\n", buffer, size);
+	printf ("%s %ld bytes\n", buffer, (long)size);
       }
       fdout = open (buffer, O_WRONLY | O_CREAT, 0666);
       if (fdout < 0) {
 	fprintf (stderr, "Unable to open %s for writing\n", buffer);
 	exit (1);
       }
-      if (write (fdout, addr, size) != size) {
-	fprintf (stderr, "Unable to write %d bytes to %s\n", size, buffer);
+      if ((size_t) write (fdout, addr, size) != size) {
+	fprintf (stderr, "Unable to write %ld bytes to %s\n", (long)size, buffer);
 	exit (1);
       }
       close (fdout);
@@ -334,5 +309,9 @@ main (int argc, char *argv[])
   if (!quiet) {
     printf ("Restored %d picture%s\n", i, i > 1 ? "s" : "");
   }
+
+  /* Free allocated memory to keep valgrind happy */
+  free(start);
+
   exit (0);
 }
